@@ -218,6 +218,10 @@ atom_explode_cat(const char *atom, const char *cat)
 		/* set to NULL if there's nothing */
 		if (ret->SLOT[0] == '\0')
 			ret->SLOT = NULL;
+
+		/* PMS 7.2: SUBSLOT defaults to SLOT when unset */
+		if (ret->SUBSLOT == NULL)
+			ret->SUBSLOT = ret->SLOT;
 	}
 
 	/* see if we have any suffix operators */
@@ -373,12 +377,15 @@ atom_clone(depend_atom *atom)
 	size_t plen = 0;
 	size_t nlen = 0;
 	size_t slen = 0;
+	size_t sslen = 0;
 	size_t rlen = 0;
 
 	if (atom->REPO != NULL)
 		rlen = strlen(atom->REPO) + 1;
 	if (atom->SLOT != NULL)
 		slen = strlen(atom->SLOT) + 1;
+	if (atom->SUBSLOT != NULL && atom->SUBSLOT != atom->SLOT)
+		sslen = strlen(atom->SUBSLOT) + 1;
 	if (atom->CATEGORY != NULL)
 		clen = strlen(atom->CATEGORY) + 1;
 	if (atom->PF != NULL)
@@ -388,7 +395,7 @@ atom_clone(depend_atom *atom)
 	if (atom->PN != NULL)
 		nlen = strlen(atom->PN) + 1;
 
-	alen = sizeof(*ret) + clen + flen + plen + nlen + rlen + slen;
+	alen = sizeof(*ret) + clen + flen + plen + nlen + rlen + slen + sslen;
 	ret = xmalloc(alen);
 	memset(ret, '\0', sizeof(*ret));
 
@@ -423,8 +430,15 @@ atom_clone(depend_atom *atom)
 		memcpy(ret->SLOT, atom->SLOT, slen);
 		p += slen;
 	}
-	if (atom->SUBSLOT > atom->SLOT && atom->SUBSLOT < (atom->SLOT + slen))
-		ret->SUBSLOT = ret->SLOT + (atom->SUBSLOT - atom->SLOT);
+	if (atom->SUBSLOT != NULL) {
+		if (atom->SUBSLOT == atom->SLOT) {  /* PMS 7.2 */
+			ret->SUBSLOT = ret->SLOT;
+		} else {
+			ret->SUBSLOT = p;
+			memcpy(ret->SUBSLOT, atom->SUBSLOT, sslen);
+			p += sslen;
+		}
+	}
 	if (atom->REPO != NULL) {
 		ret->REPO = p;
 		memcpy(ret->REPO, atom->REPO, rlen);
@@ -514,7 +528,7 @@ _atom_compare_match(int ret, atom_operator op)
  * foo-1 <NOT_EQUAL> bar-1
  */
 atom_equality
-atom_compare(const depend_atom *data, const depend_atom *query)
+atom_compare_flg(const depend_atom *data, const depend_atom *query, int flags)
 {
 	atom_operator pfx_op;
 	atom_operator sfx_op;
@@ -551,27 +565,34 @@ atom_compare(const depend_atom *data, const depend_atom *query)
 	 */
 	bl_op = query->blocker;
 	if (bl_op == ATOM_BL_ANTISLOT) {
-		/* ^perl -> match anything with a SLOT */
-		if (query->SLOT == NULL && data->SLOT == NULL)
-			return NOT_EQUAL;
-		if (query->SLOT != NULL) {
-			if (query->SUBSLOT == NULL) {
-				/* ^perl:0 -> match different SLOT */
-				if (data->SLOT == NULL ||
-						strcmp(query->SLOT, data->SLOT) == 0)
-					return NOT_EQUAL;
-			} else {
-				/* ^perl:0/5.28 -> match SLOT, but different SUBSLOT */
-				if (data->SLOT == NULL ||
-						strcmp(query->SLOT, data->SLOT) != 0)
-					return NOT_EQUAL;
-				if (data->SUBSLOT == NULL ||
-						strcmp(query->SUBSLOT, data->SUBSLOT) == 0)
-					return NOT_EQUAL;
+		/* just disable/ignore antislot op when SLOT is supposed to be
+		 * ignored */
+		if (!(flags & ATOM_COMP_NOSLOT)) {
+			/* ^perl -> match anything with a SLOT */
+			if (query->SLOT == NULL && data->SLOT == NULL)
+				return NOT_EQUAL;
+			if (query->SLOT != NULL) {
+				if (query->SUBSLOT == query->SLOT ||
+						flags & ATOM_COMP_NOSUBSLOT)
+				{
+					/* ^perl:0 -> match different SLOT */
+					if (data->SLOT == NULL ||
+							strcmp(query->SLOT, data->SLOT) == 0)
+						return NOT_EQUAL;
+				} else {
+					/* ^perl:0/5.28 -> match SLOT, but different SUBSLOT */
+					if (data->SLOT == NULL ||
+							strcmp(query->SLOT, data->SLOT) != 0)
+						return NOT_EQUAL;
+					if (!(flags & ATOM_COMP_NOSUBSLOT))
+						if (data->SUBSLOT == query->SLOT ||
+								strcmp(query->SUBSLOT, data->SUBSLOT) == 0)
+							return NOT_EQUAL;
+				}
 			}
 		}
 		bl_op = ATOM_BL_NONE;  /* ease work below */
-	} else if (query->SLOT != NULL) {
+	} else if (query->SLOT != NULL && !(flags & ATOM_COMP_NOSLOT)) {
 		/* check SLOT only when query side has it */
 		if (data->SLOT == NULL) {
 			if (bl_op == ATOM_BL_NONE)
@@ -581,9 +602,9 @@ atom_compare(const depend_atom *data, const depend_atom *query)
 				/* slot has differs */
 				if (bl_op == ATOM_BL_NONE)
 					return NOT_EQUAL;
-			} else {
-				if (query->SUBSLOT != NULL) {
-					if (data->SUBSLOT == NULL) {
+			} else if (!(flags & ATOM_COMP_NOSUBSLOT)) {
+				if (query->SUBSLOT != query->SLOT) {
+					if (data->SUBSLOT == data->SLOT) {
 						if (bl_op == ATOM_BL_NONE)
 							return NOT_EQUAL;
 					} else {
@@ -623,7 +644,7 @@ atom_compare(const depend_atom *data, const depend_atom *query)
 	}
 
 	/* check REPO, if query has it, ignore blocker stuff for this one */
-	if (query->REPO != NULL) {
+	if (query->REPO != NULL && !(flags & ATOM_COMP_NOREPO)) {
 		if (data->REPO == NULL)
 			return NOT_EQUAL;
 		if (strcmp(query->REPO, data->REPO) != 0)
@@ -753,8 +774,9 @@ atom_compare(const depend_atom *data, const depend_atom *query)
 		return EQUAL;
 
 	/* Make sure the -r# is the same. */
-	if ((sfx_op == ATOM_OP_STAR && !query->PR_int) ||
+	if ((sfx_op == ATOM_OP_STAR && query->PR_int == 0) ||
 	    pfx_op == ATOM_OP_PV_EQUAL ||
+		flags & ATOM_COMP_NOREV ||
 	    data->PR_int == query->PR_int)
 		return _atom_compare_match(EQUAL, pfx_op);
 	else if (data->PR_int < query->PR_int)
@@ -808,7 +830,8 @@ atom_to_string_r(char *buf, size_t buflen, depend_atom *a)
 	if (a->SLOT != NULL || a->slotdep != ATOM_SD_NONE)
 		off += snprintf(buf + off, buflen - off, ":%s%s%s%s",
 				a->SLOT ? a->SLOT : "",
-				a->SUBSLOT ? "/" : "", a->SUBSLOT ? a->SUBSLOT : "",
+				a->SUBSLOT && a->SUBSLOT != a->SLOT ?  "/" : "",
+				a->SUBSLOT && a->SUBSLOT != a->SLOT ? a->SUBSLOT : "",
 				atom_slotdep_str[a->slotdep]);
 	for (ud = a->usedeps; ud != NULL; ud = ud->next)
 		off += snprintf(buf + off, buflen - off, "%s%s%s%s%s",
@@ -921,7 +944,8 @@ atom_format_r(
 								HN(atom->SLOT),
 								NORM);
 				} else if (!strncmp("SUBSLOT", fmt, len)) {
-					if (showit || atom->SUBSLOT)
+					if (showit ||
+							(atom->SUBSLOT && atom->SUBSLOT != atom->SLOT))
 						append_buf(buf, buflen, "%s%s%s%s%s",
 								YELLOW,
 								connected ? "/" : "",
