@@ -30,6 +30,7 @@ char *module_name = NULL;
 int verbose = 0;
 int quiet = 0;
 int twidth;
+bool nocolor;
 char pretend = 0;
 char *portarch;
 char *portroot;
@@ -41,7 +42,8 @@ char *pkg_install_mask;
 char *binhost;
 char *pkgdir;
 char *port_tmpdir;
-char *features;
+set  *features;
+set  *ev_use;
 char *install_mask;
 DECLARE_ARRAY(overlays);
 DECLARE_ARRAY(overlay_names);
@@ -248,18 +250,31 @@ static void
 strincr_var(const char *name, const char *s, char **value, size_t *value_len)
 {
 	size_t len;
-	char *p, *nv;
-	char brace;
-
-	len = strlen(s);
-	*value = xrealloc(*value, *value_len + len + 2);
-	nv = &(*value)[*value_len];
-	if (*value_len)
-		*nv++ = ' ';
-	memcpy(nv, s, len + 1);
+	char  *p;
+	char  *nv;
+	char   brace;
+	bool   haddashstar;
 
+	/* find/skip any -* instances */
+	nv = (char *)s;
 	while ((p = strstr(nv, "-*")) != NULL)
-		memset(*value, ' ', p - *value + 2);
+		nv = p + 2;
+
+	haddashstar = nv != (char *)s;
+
+	len = strlen(nv);
+	if (haddashstar && len < *value_len) {
+		p = *value;
+		*p = '\0';  /* in case len == 0 */
+	} else if (haddashstar) {
+		p = *value = xrealloc(*value, len + 1);
+	} else {
+		*value = xrealloc(*value, *value_len + 1 + len + 1);
+		p = &(*value)[*value_len];
+		if (*value_len > 0)
+			*p++ = ' ';
+	}
+	memcpy(p, nv, len + 1);
 
 	/* This function is mainly used by the startup code for parsing
 		make.conf and stacking variables remove.
@@ -276,7 +291,6 @@ strincr_var(const char *name, const char *s, char **value, size_t *value_len)
 	*/
 
 	len = strlen(name);
-	p = nv;
 	while ((p = strchr(p, '$')) != NULL) {
 		nv = p;
 		p++;  /* skip $ */
@@ -299,6 +313,58 @@ strincr_var(const char *name, const char *s, char **value, size_t *value_len)
 	/* we should sort here */
 }
 
+static void
+setincr_var(const char *s, set **vals)
+{
+	int    i;
+	int    argc;
+	char **argv;
+	bool   ignore;
+
+	/* This tries to mimick parsing of portage envvars and the stacking
+	 * thereof.  In particular USE and FEATURES, where also negation can
+	 * happen (- prefix).  Variables are supported in form of ${v} or
+	 * $v, but there's no actual replacement happening, we just ignore
+	 * any of such forms.
+	 *  works:
+	 *		FEATURES="${FEATURES} foo"
+	 *		FEATURES="$FEATURES foo"
+	 *		FEATURES="baz bar -* foo"
+	 *
+	 *	wont work:
+	 *		FEATURES="${OTHERVAR} foo"
+	 *		FEATURES="-* ${FEATURES}"
+	 */
+
+	if (s == NULL || *s == '\0')
+		return;
+
+	/* break up input */
+	makeargv(s, &argc, &argv);
+
+	for (i = 1 /* skip executable name */; i < argc; i++) {
+		if (argv[i][0] == '-') {
+			if (*vals != NULL) {
+				/* handle negation, when the respective value isn't set, we
+				 * simply ignore/drop it */
+				if (argv[i][1] == '*') {
+					clear_set(*vals);
+				} else {
+					del_set(&argv[i][1], *vals, &ignore);
+				}
+			}
+		} else if (argv[i][0] == '$') {
+			/* detect ${var} or $var, simply ignore it completely, for
+			 * all of these should be stacked, so re-including whatever
+			 * there is shouldn't make much sense */
+		} else {
+			*vals = add_set_unique(argv[i], *vals, &ignore);
+		}
+	}
+
+	freeargv(argc, argv);
+}
+
 static env_vars *
 get_portage_env_var(env_vars *vars, const char *name)
 {
@@ -316,7 +382,11 @@ set_portage_env_var(env_vars *var, const char *value, const char *src)
 {
 	switch (var->type) {
 	case _Q_BOOL:
-		*var->value.b = 1;
+		*var->value.b = 0;
+		if (strcasecmp(value, "true") == 0 ||
+				strcasecmp(value, "yes") == 0 ||
+				strcmp(value, "1") == 0)
+			*var->value.b = 1;
 		free(var->src);
 		var->src = xstrdup(src);
 		break;
@@ -328,6 +398,7 @@ set_portage_env_var(env_vars *var, const char *value, const char *src)
 		var->src = xstrdup(src);
 		break;
 	case _Q_ISTR:
+	case _Q_ISET:
 		if (strcmp(var->src, STR_DEFAULT) != 0) {
 			size_t l = strlen(var->src) + 2 + strlen(src) + 1;
 			char *p = xmalloc(sizeof(char) * l);
@@ -341,7 +412,10 @@ set_portage_env_var(env_vars *var, const char *value, const char *src)
 			free(var->src);
 			var->src = xstrdup(src);
 		}
-		strincr_var(var->name, value, var->value.s, &var->value_len);
+		if (var->type == _Q_ISTR)
+			strincr_var(var->name, value, var->value.s, &var->value_len);
+		else
+			setincr_var(value, var->value.t);
 		break;
 	}
 }
@@ -599,7 +673,6 @@ read_portage_profile(const char *profile, env_vars vars[], set *masks)
 	read_portage_file(profile_file, PMASK_FILE, masks);
 }
 
-static bool nocolor = 0;
 env_vars vars_to_read[] = {
 #define _Q_EV(t, V, set, lset, d) \
 { \
@@ -611,10 +684,12 @@ env_vars vars_to_read[] = {
 	.default_value = d, \
 	.src = NULL, \
 },
-#define _Q_EVS(t, V, v, d) \
-	_Q_EV(t, V, .value.s = &v, .value_len = sizeof(d) - 1, d)
-#define _Q_EVB(t, V, v, d) \
-	_Q_EV(t, V, .value.b = &v, .value_len = 0, d)
+#define _Q_EVS(t, V, v, D) \
+	_Q_EV(t, V, .value.s = &v, .value_len = sizeof(D) - 1, D)
+#define _Q_EVB(t, V, v, D) \
+	_Q_EV(t, V, .value.b = &v, .value_len = 0, D)
+#define _Q_EVT(T, V, v, D) \
+	_Q_EV(T, V, .value.t = &v, .value_len = 0, D)
 
 	_Q_EVS(STR,  ROOT,                portroot,            "/")
 	_Q_EVS(STR,  ACCEPT_LICENSE,      accept_license,      "")
@@ -624,7 +699,8 @@ env_vars vars_to_read[] = {
 	_Q_EVS(ISTR, CONFIG_PROTECT,      config_protect,      "/etc")
 	_Q_EVS(ISTR, CONFIG_PROTECT_MASK, config_protect_mask, "")
 	_Q_EVB(BOOL, NOCOLOR,             nocolor,             0)
-	_Q_EVS(ISTR, FEATURES,            features,            "")
+	_Q_EVT(ISET, FEATURES,            features,            NULL)
+	_Q_EVT(ISET, USE,                 ev_use,              NULL)
 	_Q_EVS(STR,  EPREFIX,             eprefix,             CONFIG_EPREFIX)
 	_Q_EVS(STR,  EMERGE_LOG_DIR,      portlogdir,          CONFIG_EPREFIX "var/log")
 	_Q_EVS(STR,  PORTDIR,             main_overlay,        CONFIG_EPREFIX "var/db/repos/gentoo")
@@ -817,8 +893,12 @@ initialize_portage_env(void)
 	/* initialize all the properties with their default value */
 	for (i = 0; vars_to_read[i].name; ++i) {
 		var = &vars_to_read[i];
-		if (var->type != _Q_BOOL)
-			*var->value.s = xstrdup(var->default_value);
+		switch (var->type) {
+			case _Q_BOOL:  *var->value.b = var->default_value;           break;
+			case _Q_STR:
+			case _Q_ISTR:  *var->value.s = xstrdup(var->default_value);  break;
+			case _Q_ISET:  *var->value.t = (set *)var->default_value;    break;
+		}
 		var->src = xstrdup(STR_DEFAULT);
 	}
 
@@ -893,7 +973,7 @@ initialize_portage_env(void)
 		char *svar;
 
 		var = &vars_to_read[i];
-		if (var->type == _Q_BOOL)
+		if (var->type != _Q_STR)
 			continue;
 
 		while ((svar = strchr(*var->value.s, '$'))) {
@@ -1011,7 +1091,7 @@ initialize_portage_env(void)
 	}
 
 	/* Make sure ROOT always ends in a slash */
-	var = &vars_to_read[0];
+	var = &vars_to_read[0];  /* ROOT */
 	if (var->value_len == 0 || (*var->value.s)[var->value_len - 1] != '/') {
 		portroot = xrealloc(portroot, var->value_len + 2);
 		portroot[var->value_len] = '/';
@@ -1021,11 +1101,27 @@ initialize_portage_env(void)
 	if (getenv("DEBUG")) {
 		for (i = 0; vars_to_read[i].name; ++i) {
 			var = &vars_to_read[i];
-			fprintf(stderr, "%s = ", var->name);
 			switch (var->type) {
-			case _Q_BOOL: fprintf(stderr, "%i\n", *var->value.b); break;
-			case _Q_STR:
-			case _Q_ISTR: fprintf(stderr, "%s\n", *var->value.s); break;
+				case _Q_BOOL:
+					fprintf(stderr, "%s = %d\n", var->name, *var->value.b);
+					break;
+				case _Q_STR:
+				case _Q_ISTR:
+					fprintf(stderr, "%s = %s\n", var->name, *var->value.s);
+					break;
+				case _Q_ISET: {
+					DECLARE_ARRAY(vals);
+					size_t n;
+					char  *val;
+
+					fprintf(stderr, "%s = ", var->name);
+					array_set(*var->value.t, vals);
+					array_for_each(vals, n, val) {
+						fprintf(stderr, "%s ", val);
+					}
+					fprintf(stderr, "\n");
+					xarrayfree_int(vals);
+				}	break;
 			}
 		}
 	}
@@ -1038,7 +1134,7 @@ initialize_portage_env(void)
 		setenv("NOCOLOR", "true", 1);
 	} else {
 		color_remap();
-		unsetenv("NOCOLOR");
+		setenv("NOCOLOR", "false", 1);
 	}
 }
 
@@ -1055,7 +1151,11 @@ int main(int argc, char **argv)
 	bindtextdomain(argv0, CONFIG_EPREFIX "usr/share/locale");
 	textdomain(argv0);
 
+	/* note: setting nocolor here is pointless, since
+	 * initialize_portage_env is going to re-init nocolor, so make
+	 * sure we modify the default instead. */
 	twidth = 0;
+	nocolor = 0;
 	if (fstat(fileno(stdout), &st) != -1) {
 		if (!isatty(fileno(stdout))) {
 			nocolor = 1;
@@ -1066,7 +1166,10 @@ int main(int argc, char **argv)
 			if (ioctl(0, TIOCGWINSZ, &winsz) == 0 && winsz.ws_col > 0)
 				twidth = (int)winsz.ws_col;
 		}
+	} else {
+		nocolor = 1;
 	}
+	vars_to_read[7].default_value = (char *)nocolor;  /* NOCOLOR */
 
 	initialize_portage_env();
 	optind = 0;
