@@ -437,6 +437,7 @@ static int do_emerge_log(
 	size_t i;
 	size_t parallel_emerge = 0;
 	bool all_atoms = false;
+	bool emerge_line;
 	char afmt[BUFSIZ];
 	struct pkg_match *pkg;
 	struct pkg_match *pkgw;
@@ -544,14 +545,21 @@ static int do_emerge_log(
 
 		tstart = atol(buf);
 
+		emerge_line = false;
+		if (((flags->do_running ||
+			  (flags->show_emerge && verbose)) &&
+			 strpfx(p, "  *** emerge ") == 0) ||
+			(flags->do_running &&
+			 (strpfx(p, "  *** exiting ") == 0 ||
+			  strpfx(p, "  *** terminating.") == 0)))
+		{
+			emerge_line = true;
+		}
+
 		/* keeping track of parallel merges needs to be done before
 		 * applying dates, for a subset of the log might show emerge
 		 * finished without knowledge of another instance */
-		if (flags->do_running &&
-				(strpfx(p, "  *** emerge ") == 0 ||
-				 strpfx(p, "  *** exiting ") == 0 ||
-				 strpfx(p, "  *** terminating.") == 0))
-		{
+		if (emerge_line && flags->do_running) {
 			if (p[7] == 'm') {
 				parallel_emerge++;
 			} else if (parallel_emerge > 0) {
@@ -593,8 +601,7 @@ static int do_emerge_log(
 			continue;
 
 		/* are we interested in this line? */
-		if (flags->show_emerge && verbose && (strpfx(p, "  *** emerge ") == 0))
-		{
+		if (emerge_line && flags->show_emerge && verbose) {
 			char shortopts[8];  /* must hold as many opts converted below */
 			int numopts = 0;
 
@@ -651,14 +658,11 @@ static int do_emerge_log(
 
 				if (strpfx(q, "--") == 0) {
 					printf(" %s%s%s", GREEN, q, NORM);
-				} else if (strcmp(q, "@world") == 0 ||
-						strcmp(q, "@system") == 0)
+				} else if (q[0] == '@' ||
+						   strcmp(q, "world") == 0 ||
+						   strcmp(q, "system") == 0)
 				{
 					printf(" %s%s%s", YELLOW, q, NORM);
-				} else if (strcmp(q, "world") == 0 ||
-						strcmp(q, "system") == 0)
-				{
-					printf(" %s@%s%s", YELLOW, q, NORM);
 				} else {
 					/* this should be an atom */
 					atom = atom_explode(q);
@@ -753,15 +757,17 @@ static int do_emerge_log(
 				/* see if we need this atom */
 				atomw = NULL;
 				if (atomset == NULL) {
-					int orev = atom->PR_int;
-					if (flags->do_predict)
-						atom->PR_int = 0;  /* allow matching a revision */
+					/* match without revisions when we try to predict,
+					 * such that our set remains rich enough to cover
+					 * various predictions */
 					array_for_each(atoms, i, atomw) {
-						if (atom_compare(atom, atomw) == EQUAL)
+						if (atom_compare_flg(atom, atomw,
+									flags->do_predict
+									? ATOM_COMP_NOREV
+									: ATOM_COMP_DEFAULT) == EQUAL)
 							break;
 						atomw = NULL;
 					}
-					atom->PR_int = orev;
 				} else {
 					snprintf(afmt, sizeof(afmt), "%s/%s",
 							atom->CATEGORY, atom->PN);
@@ -1229,11 +1235,8 @@ static int do_emerge_log(
 
 			array_for_each(avgs, j, pkg) {
 				if (pkgstate == P_INIT) {
-					int orev = pkg->atom->PR_int;
 					atom_equality eq;
-					pkg->atom->PR_int = 0;
-					eq = atom_compare(pkg->atom, atom);
-					pkg->atom->PR_int = orev;
+					eq = atom_compare_flg(pkg->atom, atom, ATOM_COMP_NOREV);
 					switch (eq) {
 						case EQUAL:
 							/* version-less atoms equal any versioned
